@@ -2,27 +2,22 @@
 
 use crate::media_type::MediaType;
 use crate::module_specifier::ModuleSpecifier;
-#[cfg(target_arch = "wasm32")]
-use crate::module_specifier::EMPTY_SPECIFIER;
 
 use anyhow::Result;
 use lazy_static::lazy_static;
 use regex::Match;
 use regex::Regex;
 use serde::Serialize;
+use std::collections::HashMap;
 use std::fmt;
-use std::rc::Rc;
+use std::sync::Arc;
 use swc_common::comments::Comment;
 use swc_common::comments::SingleThreadedComments;
 use swc_common::BytePos;
-use swc_common::FileName;
-use swc_common::SourceMap;
 use swc_common::Spanned;
 use swc_ecmascript::ast::Module;
-use swc_ecmascript::dep_graph::analyze_dependencies;
 use swc_ecmascript::dep_graph::DependencyDescriptor;
 use swc_ecmascript::dep_graph::DependencyKind;
-use swc_ecmascript::parser;
 use swc_ecmascript::parser::lexer::Lexer;
 use swc_ecmascript::parser::EsConfig;
 use swc_ecmascript::parser::JscTarget;
@@ -30,6 +25,7 @@ use swc_ecmascript::parser::Parser;
 use swc_ecmascript::parser::StringInput;
 use swc_ecmascript::parser::Syntax;
 use swc_ecmascript::parser::TsConfig;
+use text_lines::TextLines;
 
 static TARGET: JscTarget = JscTarget::Es2021;
 
@@ -79,8 +75,8 @@ fn get_ts_config(tsx: bool, dts: bool) -> TsConfig {
   }
 }
 
-impl<'a> From<&'a MediaType> for Syntax {
-  fn from(media_type: &'a MediaType) -> Self {
+impl<'a> From<MediaType> for Syntax {
+  fn from(media_type: MediaType) -> Self {
     match media_type {
       MediaType::JavaScript => Self::Es(get_es_config(false)),
       MediaType::Jsx => Self::Es(get_es_config(true)),
@@ -94,16 +90,19 @@ impl<'a> From<&'a MediaType> for Syntax {
 
 #[derive(Debug, Clone, PartialEq, Eq, Serialize)]
 pub struct Position {
+  /// The 0-indexed line index.
   pub line: usize,
+  /// The 0-indexed character index.
   pub character: usize,
 }
 
 impl Position {
-  fn from_byte_pos(sm: &Rc<SourceMap>, pos: BytePos) -> Self {
-    let loc = sm.lookup_char_pos(pos);
-    Self {
-      line: loc.line - 1,
-      character: loc.col_display,
+  pub fn from_pos(text_lines: &TextLines, pos: BytePos) -> Self {
+    let line_and_column_index =
+      text_lines.line_and_column_index(pos.0 as usize);
+    Position {
+      line: line_and_column_index.line_index,
+      character: line_and_column_index.column_index,
     }
   }
 }
@@ -130,27 +129,24 @@ impl Default for Range {
 }
 
 impl Range {
-  fn from_comment_match(
-    comment: &Comment,
-    sm: &Rc<SourceMap>,
-    m: &Match,
-  ) -> Self {
-    Self {
-      start: Position::from_byte_pos(
-        sm,
-        comment.span.lo + BytePos((m.start() + 1) as u32),
-      ),
-      end: Position::from_byte_pos(
-        sm,
-        comment.span.lo + BytePos((m.end() + 1) as u32),
-      ),
+  pub(crate) fn from_span(
+    parsed_ast: &dyn ParsedAst,
+    span: &swc_common::Span,
+  ) -> Range {
+    Range {
+      start: parsed_ast.get_position(span.lo),
+      end: parsed_ast.get_position(span.hi),
     }
   }
 
-  pub fn from_span(sm: &Rc<SourceMap>, span: &swc_common::Span) -> Self {
+  fn from_comment_match(
+    comment: &Comment,
+    m: &Match,
+    get_position: impl Fn(BytePos) -> Position,
+  ) -> Self {
     Self {
-      start: Position::from_byte_pos(sm, span.lo),
-      end: Position::from_byte_pos(sm, span.hi),
+      start: get_position(comment.span.lo + BytePos((m.start() + 1) as u32)),
+      end: get_position(comment.span.lo + BytePos((m.end() + 1) as u32)),
     }
   }
 }
@@ -193,44 +189,6 @@ impl fmt::Display for Location {
   }
 }
 
-#[cfg(not(target_arch = "wasm32"))]
-impl From<swc_common::Loc> for Location {
-  fn from(loc: swc_common::Loc) -> Self {
-    let specifier = match &loc.file.name {
-      FileName::Real(path) => ModuleSpecifier::from_file_path(path).unwrap(),
-      FileName::Custom(input) => ModuleSpecifier::parse(input).unwrap(),
-      _ => unreachable!("invalid specifier"),
-    };
-
-    Location {
-      specifier,
-      position: Position {
-        line: loc.line,
-        character: loc.col_display,
-      },
-    }
-  }
-}
-
-#[cfg(target_arch = "wasm32")]
-impl From<swc_common::Loc> for Location {
-  fn from(loc: swc_common::Loc) -> Self {
-    let specifier = match &loc.file.name {
-      FileName::Real(_) => ModuleSpecifier::parse(EMPTY_SPECIFIER).unwrap(),
-      FileName::Custom(input) => ModuleSpecifier::parse(input).unwrap(),
-      _ => unreachable!("invalid specifier"),
-    };
-
-    Location {
-      specifier,
-      position: Position {
-        line: loc.line,
-        character: loc.col_display,
-      },
-    }
-  }
-}
-
 pub enum TypeScriptReference {
   Path(String, Range),
   Types(String, Range),
@@ -250,31 +208,45 @@ impl fmt::Display for Diagnostic {
   }
 }
 
-impl Diagnostic {
-  fn from_parser_error(err: parser::error::Error, sm: &Rc<SourceMap>) -> Self {
-    Self {
-      location: sm.lookup_char_pos(err.span().lo).into(),
-      message: err.into_kind().msg().to_string(),
-    }
-  }
-}
+/// A parsed module with comments.
+pub trait ParsedAst {
+  /// Gets the module specifier of the module.
+  fn specifier(&self) -> &ModuleSpecifier;
+  /// Gets the media type of the module.
+  fn media_type(&self) -> MediaType;
+  /// Gets the text content of the module.
+  fn source(&self) -> Arc<String>;
+  /// Gets the comments found in the source file.
+  fn comments(&self) -> &dyn swc_common::comments::Comments;
+  /// Gets the parsed module—AST.
+  fn module(&self) -> &swc_ecmascript::ast::Module;
+  /// Gets a position from the specified byte position.
+  /// Note: This should/will panic when the byte position is
+  // outside the bounds of the source file.
+  fn get_position(&self, pos: BytePos) -> Position;
 
-pub(crate) struct ParsedModule {
-  comments: SingleThreadedComments,
-  pub leading_comments: Vec<Comment>,
-  module: Module,
-  source_map: Rc<SourceMap>,
-}
-
-impl ParsedModule {
-  pub fn analyze_dependencies(&self) -> Vec<DependencyDescriptor> {
-    analyze_dependencies(&self.module, &self.comments)
-      .into_iter()
-      .filter(|desc| desc.kind != DependencyKind::Require)
-      .collect()
+  /// Get the module's leading comments, where triple slash directives might
+  /// be located.
+  fn get_leading_comments(&self) -> Vec<Comment> {
+    self
+      .comments()
+      .get_leading(self.module().span.lo)
+      .unwrap_or_else(Vec::new)
   }
 
-  pub fn analyze_deno_types(
+  /// Gets all the dependencies of this module.
+  fn analyze_dependencies(&self) -> Vec<DependencyDescriptor> {
+    swc_ecmascript::dep_graph::analyze_dependencies(
+      self.module(),
+      self.comments(),
+    )
+    .into_iter()
+    .filter(|desc| desc.kind != DependencyKind::Require)
+    .collect()
+  }
+
+  /// Searches comments for any `@deno-types` compiler hints.
+  fn analyze_deno_types(
     &self,
     desc: &DependencyDescriptor,
   ) -> Option<(String, Range)> {
@@ -283,27 +255,30 @@ impl ParsedModule {
     if let Some(m) = captures.get(1) {
       Some((
         m.as_str().to_string(),
-        Range::from_comment_match(comment, &self.source_map, &m),
+        Range::from_comment_match(comment, &m, |pos| self.get_position(pos)),
       ))
     } else if let Some(m) = captures.get(2) {
       Some((
         m.as_str().to_string(),
-        Range::from_comment_match(comment, &self.source_map, &m),
+        Range::from_comment_match(comment, &m, |pos| self.get_position(pos)),
       ))
     } else {
       unreachable!("Unexpected captures from deno types regex")
     }
   }
 
-  pub fn analyze_ts_references(&self) -> Vec<TypeScriptReference> {
+  /// Searches comments for any triple slash references.
+  fn analyze_ts_references(&self) -> Vec<TypeScriptReference> {
     let mut references = Vec::new();
-    for comment in &self.leading_comments {
+    for comment in self.get_leading_comments().iter() {
       if TRIPLE_SLASH_REFERENCE_RE.is_match(&comment.text) {
         if let Some(captures) = PATH_REFERENCE_RE.captures(&comment.text) {
           let m = captures.get(1).unwrap();
           references.push(TypeScriptReference::Path(
             m.as_str().to_string(),
-            Range::from_comment_match(comment, &self.source_map, &m),
+            Range::from_comment_match(comment, &m, |pos| {
+              self.get_position(pos)
+            }),
           ));
         } else if let Some(captures) =
           TYPES_REFERENCE_RE.captures(&comment.text)
@@ -311,45 +286,154 @@ impl ParsedModule {
           let m = captures.get(1).unwrap();
           references.push(TypeScriptReference::Types(
             m.as_str().to_string(),
-            Range::from_comment_match(comment, &self.source_map, &m),
+            Range::from_comment_match(comment, &m, |pos| {
+              self.get_position(pos)
+            }),
           ));
         }
       }
     }
     references
   }
+}
 
-  pub fn range_from_span(&self, span: &swc_common::Span) -> Range {
-    Range::from_span(&self.source_map, span)
+/// Parses modules.
+pub trait AstParser {
+  /// Parses the provided information to a `ParsedAst` and returns a reference.
+  fn parse(
+    &mut self,
+    specifier: &ModuleSpecifier,
+    source: Arc<String>,
+    media_type: MediaType,
+  ) -> Result<&dyn ParsedAst, Diagnostic>;
+}
+
+pub struct DefaultParsedAst {
+  specifier: ModuleSpecifier,
+  media_type: MediaType,
+  source: Arc<String>,
+  comments: SingleThreadedComments,
+  module: Module,
+  text_lines: TextLines,
+}
+
+impl ParsedAst for DefaultParsedAst {
+  fn specifier(&self) -> &ModuleSpecifier {
+    &self.specifier
+  }
+
+  fn media_type(&self) -> MediaType {
+    self.media_type
+  }
+
+  fn source(&self) -> Arc<String> {
+    self.source.clone()
+  }
+
+  fn comments(&self) -> &dyn swc_common::comments::Comments {
+    &self.comments
+  }
+
+  fn module(&self) -> &swc_ecmascript::ast::Module {
+    &self.module
+  }
+
+  fn get_position(&self, pos: BytePos) -> Position {
+    Position::from_pos(&self.text_lines, pos)
   }
 }
 
-pub(crate) fn parse(
+/// An implementation of `AstParser` that stores the parsed ASTs.
+#[derive(Default)]
+pub struct CapturingAstParser {
+  modules: HashMap<ModuleSpecifier, DefaultParsedAst>,
+}
+
+#[cfg(feature = "rust")]
+impl CapturingAstParser {
+  pub fn new() -> Self {
+    Self {
+      modules: HashMap::new(),
+    }
+  }
+
+  /// Gets an AST by a module specifier if it was previously parsed.
+  pub fn get_ast(
+    &self,
+    specifier: &ModuleSpecifier,
+  ) -> Option<&DefaultParsedAst> {
+    self.modules.get(specifier)
+  }
+}
+
+impl AstParser for CapturingAstParser {
+  fn parse(
+    &mut self,
+    specifier: &ModuleSpecifier,
+    source: Arc<String>,
+    media_type: MediaType,
+  ) -> Result<&dyn ParsedAst, Diagnostic> {
+    let ast = parse_default_ast(specifier, source, media_type)?;
+
+    self.modules.insert(specifier.clone(), ast);
+
+    Ok(self.modules.get(specifier).unwrap())
+  }
+}
+
+/// The default implementation of `AstParser` used by this crate.
+#[derive(Default)]
+pub struct DefaultAstParser {
+  module: Option<DefaultParsedAst>,
+}
+
+impl DefaultAstParser {
+  pub fn new() -> Self {
+    Self { module: None }
+  }
+}
+
+impl AstParser for DefaultAstParser {
+  fn parse(
+    &mut self,
+    specifier: &ModuleSpecifier,
+    source: Arc<String>,
+    media_type: MediaType,
+  ) -> Result<&dyn ParsedAst, Diagnostic> {
+    let ast = parse_default_ast(specifier, source, media_type)?;
+
+    self.module.replace(ast);
+
+    Ok(self.module.as_ref().unwrap())
+  }
+}
+
+fn parse_default_ast(
   specifier: &ModuleSpecifier,
-  source: &str,
-  media_type: &MediaType,
-) -> Result<ParsedModule, Diagnostic> {
-  let source_map = Rc::new(SourceMap::default());
-  let source_file = source_map.new_source_file(
-    FileName::Custom(specifier.as_str().to_string()),
-    source.to_string(),
-  );
-  let input = StringInput::from(&*source_file);
+  source: Arc<String>,
+  media_type: MediaType,
+) -> Result<DefaultParsedAst, Diagnostic> {
+  let text_lines = TextLines::new(&source);
+  let input =
+    StringInput::new(&source, BytePos(0), BytePos(source.len() as u32));
   let comments = SingleThreadedComments::default();
   let lexer = Lexer::new(media_type.into(), TARGET, input, Some(&comments));
   let mut parser = Parser::new_from(lexer);
-  let sm = &source_map;
-  let module = parser
-    .parse_module()
-    .map_err(|err| Diagnostic::from_parser_error(err, sm))?;
-  let leading_comments =
-    comments.with_leading(module.span.lo, |comments| comments.to_vec());
+  let module = parser.parse_module().map_err(|err| Diagnostic {
+    location: Location {
+      specifier: specifier.clone(),
+      position: Position::from_pos(&text_lines, err.span().lo),
+    },
+    message: err.into_kind().msg().to_string(),
+  })?;
 
-  Ok(ParsedModule {
+  Ok(DefaultParsedAst {
+    specifier: specifier.clone(),
+    media_type,
+    source,
     comments,
-    leading_comments,
     module,
-    source_map,
+    text_lines,
   })
 }
 
@@ -361,7 +445,8 @@ mod tests {
   fn test_parse() {
     let specifier =
       ModuleSpecifier::parse("file:///a/test.ts").expect("bad specifier");
-    let source = r#"import {
+    let source = Arc::new(
+      r#"import {
       A,
       B,
       C,
@@ -377,19 +462,23 @@ mod tests {
     import React from "https://cdn.skypack.dev/react";
 
     const a = await import("./a.ts");
-    "#;
-    let result = parse(&specifier, source, &MediaType::TypeScript);
+    "#
+      .to_string(),
+    );
+    let mut parser = DefaultAstParser::new();
+    let result = parser.parse(&specifier, source, MediaType::TypeScript);
     assert!(result.is_ok());
-    let parsed_module = result.unwrap();
-    assert_eq!(parsed_module.analyze_dependencies().len(), 6);
-    assert_eq!(parsed_module.analyze_ts_references().len(), 0);
+    let parsed_ast = result.unwrap();
+    assert_eq!(parsed_ast.analyze_dependencies().len(), 6);
+    assert_eq!(parsed_ast.analyze_ts_references().len(), 0);
   }
 
   #[test]
   fn test_analyze_dependencies() {
     let specifier =
       ModuleSpecifier::parse("file:///a/test.ts").expect("bad specifier");
-    let source = r#"
+    let source = Arc::new(
+      r#"
     import * as a from "./a.ts";
     import "./b.ts";
     import { c } from "./c.ts";
@@ -401,14 +490,17 @@ mod tests {
 
     import type { i } from "./i.d.ts";
     export type { j } from "./j.d.ts";
-    "#;
-    let result = parse(&specifier, source, &MediaType::TypeScript);
+    "#
+      .to_string(),
+    );
+    let mut parser = DefaultAstParser::new();
+    let result = parser.parse(&specifier, source, MediaType::TypeScript);
     assert!(result.is_ok());
-    let parsed_module = result.unwrap();
-    let dependencies = parsed_module.analyze_dependencies();
+    let parsed_ast = result.unwrap();
+    let dependencies = parsed_ast.analyze_dependencies();
     assert_eq!(dependencies.len(), 10);
     assert_eq!(dependencies[0].specifier.to_string(), "./a.ts");
-    let range = parsed_module.range_from_span(&dependencies[0].specifier_span);
+    let range = Range::from_span(parsed_ast, &dependencies[0].specifier_span);
     assert_eq!(
       range,
       Range {
@@ -424,7 +516,7 @@ mod tests {
     );
     assert!(!dependencies[0].is_dynamic);
     assert_eq!(dependencies[1].specifier.to_string(), "./b.ts");
-    let range = parsed_module.range_from_span(&dependencies[1].specifier_span);
+    let range = Range::from_span(parsed_ast, &dependencies[1].specifier_span);
     assert_eq!(
       range,
       Range {
